package com.stellvoy.user.controller;

import com.stellvoy.api.client.learning.LearningClient;
import com.stellvoy.api.dto.leanring.LearningInterestDTO;
import com.stellvoy.api.dto.leanring.LearningMedalDTO;
import com.stellvoy.api.dto.user.LoginFormDTO;
import com.stellvoy.api.dto.user.NewUserDTO;
import com.stellvoy.common.domain.dto.LoginUserDTO;
import com.stellvoy.common.exceptions.BadRequestException;
import com.stellvoy.common.utils.BeanUtils;
import com.stellvoy.user.constants.UserErrorInfo;
import com.stellvoy.user.domain.dto.UpdateEmailDTO;
import com.stellvoy.user.domain.dto.UpdatePasswordDTO;
import com.stellvoy.user.domain.dto.UserTaskDTO;
import com.stellvoy.user.domain.dto.VerifyCodeDTO;
import com.stellvoy.user.domain.po.*;
import com.stellvoy.user.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.beans.Transient;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/users")
public class UserController {

    @Autowired
    private IUserService userService;
    @Autowired
    private ICodeService codeService;
    @Autowired
    private UserMedalService userMedalService;
    @Autowired
    private UserInterestService userInterestService;
    @Autowired
    private LearningClient learningClient;

    /**
     * 根据id查询用户信息
     */
    @GetMapping("/{id}")
    public NewUserDTO queryUserById(@PathVariable("id") Long id) {
        User one = userService.lambdaQuery().eq(User::getId, id).one();
        return BeanUtils.copyBean(one, NewUserDTO.class);
    }
    /**
     * 根据id集合查询用户信息
     */
    @GetMapping("/list/{ids}")
    public List<NewUserDTO> queryUserByIds(@PathVariable("ids") List<Long> ids) {
        List<User> userList = userService.lambdaQuery().in(User::getId, ids).list();
        return BeanUtils.copyList(userList, NewUserDTO.class);
    }
    /**
     * 登录结构
     * @param loginDTO 登录表单
     * @param isStaff 是否是后台登录
     * @return 登录用户信息
     */
    @PostMapping("/detail/{isStaff}")
    public LoginUserDTO queryUserDetail(
            @Valid @RequestBody LoginFormDTO loginDTO, @PathVariable("isStaff") boolean isStaff) {
        return userService.queryUserDetail(loginDTO, isStaff);
    }

    /**
     * 获取手机登录验证码
     */
    @GetMapping("/number/{phone}")
    public void getNumber(
            @PathVariable("phone") String cellPhone) {
        codeService.sendVerifyCode(cellPhone);
    }


    @GetMapping("/list/ids")
    public Long exchangeUserIdWithPhone(@RequestParam("phone") String phone) {
        User user = userService
                .lambdaQuery().eq(User::getCellPhone, phone).one();
        if (user == null) {
            throw new BadRequestException(UserErrorInfo.Msg.USER_ID_NOT_EXISTS);
        }
        return user.getId();
    }

    /**
     * 根据用户ID查询全部勋章
     */
    @GetMapping("/medal/{userId}")
    public List<LearningMedalDTO> queryUserMedal(@PathVariable("userId") Long userId) {
        List<Long> medalIds = userMedalService.lambdaQuery()
                .eq(UserMedal::getUserId, userId)
                .list()
                .stream()
                .map(UserMedal::getMedalId)
                .collect(Collectors.toList());
        List<LearningMedalDTO> dtoList = learningClient.queryMedal();
        for (LearningMedalDTO learningMedalDTO : dtoList) {
            if(medalIds.contains(learningMedalDTO.getId())){
                learningMedalDTO.setType(1);
            }
        }
        return dtoList;
    }

    /**
     * 根据用户ID查询全部兴趣标签
     */
    @GetMapping("/interest/{userId}")
    public List<LearningInterestDTO> queryUserInterest(@PathVariable("userId") Long userId) {
        List<Long> interestIds = userInterestService.lambdaQuery()
                .eq(UserInterest::getUserId, userId)
                .list()
                .stream()
                .map(UserInterest::getInterestId)
                .collect(Collectors.toList());
        List<LearningInterestDTO> dtoList = learningClient.queryInterest();
        for (LearningInterestDTO learningInterestDTO : dtoList) {
            if(interestIds.contains(learningInterestDTO.getId())){
                learningInterestDTO.setType(1);
            }
        }
        return dtoList;
    }

    /**
     * 实名认证
     */
    @PostMapping("/verify/{userId}")
    public void verifyCode(@RequestBody VerifyCodeDTO dto, @PathVariable("userId") Long userId) {
        userService.verifyCode(dto,userId);
    }

    /**
     * 修改登录密码
     */
    @PostMapping("/password/{userId}")
    public void updatePassword(@RequestBody UpdatePasswordDTO dto, @PathVariable("userId") Long userId) {
        userService.updatePassword(dto,userId);
    }

    /**
     * 绑定邮箱
     */
    @PostMapping("/email/{userId}")
    public void updateEmail(@RequestBody UpdateEmailDTO dto, @PathVariable("userId") Long userId) {
        userService.updateEmail(dto,userId);
    }

    /**
     * 邮箱验证码
     */
    @GetMapping("/email")
    public void sendEmailCode(@RequestParam String email) {
        codeService.sendEmailCode(email);
    }

    /**
     * 修改用户兴趣标签
     */
    @Transient
    @PostMapping("/updateInterest/{userId}")
    public void updateInterest(@RequestBody List<Long> interestIds, @PathVariable("userId") Long userId) {
        // 先删除指定用户的所有兴趣标签记录
        userInterestService.lambdaUpdate()
                .eq(UserInterest::getUserId, userId)
                .remove();

        // 批量插入新的兴趣标签记录
        if (interestIds != null && !interestIds.isEmpty()) {
            List<UserInterest> newUserInterests = interestIds.stream()
                    .map(interestId -> {
                        UserInterest userInterest = new UserInterest();
                        userInterest.setUserId(userId);
                        userInterest.setInterestId(interestId);
                        return userInterest;
                    })
                    .collect(Collectors.toList());

            userInterestService.saveBatch(newUserInterests);
        }
    }

    /**
     *修改isNew
     */
    @PostMapping("/isNew/{userId}")
    public void isNew(@PathVariable("userId") Long userId) {
        userService.lambdaUpdate()
                .eq(User::getId, userId)
                .set(User::getIsNew, 1)
                .update();
    }

    /**
     * 统计用户任务数量
     * @param userId
     * @return
     */
    @GetMapping("/stats/{userId}")
    public UserTaskDTO getStats(@PathVariable Long userId) {
        return userService.getStats(userId);
    }
    /**
     * 上传个人简历
     */
    @PostMapping("/resume/{userId}")
    public void uploadResume(@RequestBody String resume, @PathVariable("userId") Long userId) {
        if (resume == null || resume.length() < 2) {
            throw new IllegalArgumentException("简历内容无效");
        }
        String trimmedResume = resume.substring(1, resume.length() - 1);
        userService.lambdaUpdate().eq(User::getId, userId)
                .set(User::getResume, trimmedResume)
                .update();
    }
}
