package com.stellvoy.user.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.stellvoy.api.client.auth.AuthClient;
import com.stellvoy.api.client.task.TaskClient;
import com.stellvoy.api.dto.auth.RoleDTO;
import com.stellvoy.api.dto.task.NewTaskDTO;
import com.stellvoy.api.dto.user.LoginFormDTO;
import com.stellvoy.api.dto.user.UserDTO;
import com.stellvoy.common.domain.dto.LoginUserDTO;
import com.stellvoy.common.enums.UserType;
import com.stellvoy.common.exceptions.BadRequestException;
import com.stellvoy.common.exceptions.ForbiddenException;
import com.stellvoy.common.exceptions.UnauthorizedException;
import com.stellvoy.common.utils.AssertUtils;
import com.stellvoy.common.utils.BeanUtils;
import com.stellvoy.common.utils.StringUtils;
import com.stellvoy.common.utils.UserContext;
import com.stellvoy.user.domain.dto.*;
import com.stellvoy.user.domain.po.User;
import com.stellvoy.user.domain.po.UserDetail;
import com.stellvoy.user.domain.po.UserTask;
import com.stellvoy.user.domain.vo.UserDetailVO;
import com.stellvoy.user.mapper.UserMapper;
import com.stellvoy.user.service.ICodeService;
import com.stellvoy.user.service.IUserDetailService;
import com.stellvoy.user.service.IUserService;
import com.stellvoy.user.service.UserTaskService;
import com.stellvoy.user.utils.AliyunIdCardVerifier;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.Month;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.stellvoy.user.constants.UserConstants.*;
import static com.stellvoy.user.constants.UserErrorInfo.Msg.*;


/**
 * <p>
 * 学员用户表 服务实现类
 * </p>
 */
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private ICodeService codeService;
    @Autowired
    private AuthClient authClient;
    @Autowired
    private IUserDetailService detailService;
    @Autowired
    private AliyunIdCardVerifier aliyunIdCardVerifier;
    @Autowired
    private StringRedisTemplate stringRedisTemplate;
    @Autowired
    private UserTaskService userTaskService;
    @Autowired
    private TaskClient taskClient;
    @Override
    public LoginUserDTO queryUserDetail(LoginFormDTO loginDTO, boolean isStaff) {
        // 1.判断登录方式
        Integer type = loginDTO.getType();
        User user = null;
        // 2.手机号和密码登录
        if (type == 1) {
            user = loginByPw(loginDTO);
        }
        // 3.验证码登录
        if (type == 2) {
            user = loginByVerifyCode(loginDTO.getCellPhone(), loginDTO.getPassword());
        }
        // 4.错误的登录方式
        if (user == null) {
            throw new BadRequestException(ILLEGAL_LOGIN_TYPE);
        }
        // 6.封装返回
        LoginUserDTO userDTO = new LoginUserDTO();
        userDTO.setUserId(user.getId());
        userDTO.setRoleId(handleRoleId(user));
        return userDTO;
    }

    @Override
    public void resetPassword(Long userId) {
        User user = new User();
        user.setId(userId);
        user.setPassword(passwordEncoder.encode(DEFAULT_PASSWORD));
        AssertUtils.isTrue(updateById(user), USER_ID_NOT_EXISTS);
    }

    @Override
    public UserDetailVO myInfo() {
        // 1.获取登录用户id
        Long userId = UserContext.getUser();
        if (userId == null) {
            return null;
        }
        // 2.查询用户
        UserDetail userDetail = detailService.queryById(userId);
        AssertUtils.isNotNull(userDetail, USER_ID_NOT_EXISTS);
        // 3.封装vo
        UserType type = userDetail.getType();
        // 3.1.基本信息
        UserDetailVO vo = BeanUtils.toBean(userDetail, UserDetailVO.class);
        // 3.2.详情信息
        switch (type) {
            case STAFF:
                RoleDTO roleDTO = authClient.queryRoleById(userDetail.getRoleId());
                vo.setRoleName(roleDTO == null ? "" : roleDTO.getName());
                break;
            case STUDENT:
                vo.setRoleName(STUDENT_ROLE_NAME);
                break;
            case TEACHER:
                vo.setRoleName(TEACHER_ROLE_NAME);
                break;
            default:
                break;
        }
        return vo;
    }

    @Override
    public void addUserByPhone(User user, String code) {
        // 1.验证码校验
        codeService.verifyCode(user.getCellPhone(), code);
        // 2.判断手机号是否存在
        Long count = lambdaQuery().eq(User::getCellPhone, user.getCellPhone()).count();
        if (count > 0) {
            throw new BadRequestException(PHONE_ALREADY_EXISTS);
        }
        // 3.加密密码
        user.setPassword(passwordEncoder.encode(user.getPassword()));
        // 4.新增
        user.setUsername(user.getCellPhone());
        save(user);
    }

    @Override
    public void updatePasswordByPhone(String cellPhone, String code, String password) {
        // 1.验证码校验
        codeService.verifyCode(cellPhone, code);
        // 2.查询用户
        User oldUser = lambdaQuery().eq(User::getCellPhone, cellPhone).one();
        if (oldUser == null) {
            // 手机号不存在
            throw new BadRequestException(PHONE_NOT_EXISTS);
        }
        // 2.修改密码
        User user = new User();
        user.setId(user.getId());
        user.setPassword(passwordEncoder.encode(password));
        updateById(user);
    }

    public void updatePhoneById(Long id, String cellPhone) {
        // 1.1.判断是否需要修改手机号
        if (StringUtils.isNotBlank(cellPhone)) {
            // 1.2.需要修改，封装数据
            User user = new User();
            user.setId(id);
            user.setUsername(cellPhone);
            user.setCellPhone(cellPhone);
            // 1.3.修改
            updateById(user);
        }
    }

    @Override
    @Transactional
    public Long saveUser(UserDTO userDTO) {
        // 1.保存用户基本信息
        User user = new User();
        user.setPassword(passwordEncoder.encode(DEFAULT_PASSWORD));
        user.setCellPhone(userDTO.getCellPhone());
        user.setUsername(userDTO.getCellPhone());
        save(user);
        // 2.新增详情
        UserDetail detail = BeanUtils.toBean(userDTO, UserDetail.class);
        detail.setId(user.getId());
        detailService.save(detail);
        return user.getId();
    }

    @Override
    @Transactional
    public void updateUser(UserDTO userDTO) {
        // 1.如果传递了手机号，则修改手机号
        String cellphone = userDTO.getCellPhone();
        if(StringUtils.isNotBlank(cellphone)){
            User user = new User();
            user.setId(userDTO.getId());
            user.setCellPhone(cellphone);
            user.setUsername(cellphone);
            updateById(user);
        }
        // 2.修改详情
        UserDetail detail = BeanUtils.toBean(userDTO, UserDetail.class);
        detail.setType(null);
        detailService.updateById(detail);
    }

    @Override
    public void updateUserWithPassword(UserFormDTO userDTO) {
        // 1.尝试更新密码
        String pw = userDTO.getPassword();
        String oldPw = userDTO.getOldPassword();
        if(StringUtils.isNotBlank(pw) && StringUtils.isNotBlank(pw)) {
            Long userId = UserContext.getUser();
            // 1.1.查询用户
            User user = getById(userId);
            // 1.2.校验
            if (user == null) {
                throw new UnauthorizedException(USER_ID_NOT_EXISTS);
            }
            // 1.3.校验密码
            if (!passwordEncoder.matches(oldPw, user.getPassword())) {
                // 密码不一致
                throw new UnauthorizedException(INVALID_UN_OR_PW);
            }
            // 1.4.修改密码
            user = new User();
            user.setId(userId);
            user.setPassword(passwordEncoder.encode(pw));
            updateById(user);
        }
        // 2.更新用户详情
        UserDetail detail = BeanUtils.toBean(userDTO, UserDetail.class);
        detail.setRoleId(null);
        detail.setType(null);
        detailService.updateById(detail);
    }

    /**
     * 手机号注册
     */
    public User phoneRegister(UserRegisterDTO dto) {
        // 1.校验验证码
        codeService.verifyCode(dto.getCellPhone(), dto.getCode());
        // 2.根据手机号查询
        User one = lambdaQuery().eq(User::getCellPhone, dto.getCellPhone()).one();
        if (one != null) {
            throw new BadRequestException("手机号已经被注册");
        }
        User user = BeanUtils.copyBean(dto, User.class);
        user.setStatus(1);
//        user.setPassword(passwordEncoder.encode(dto.getPassword()));
        user.setPassword("123456");
        user.setCreateTime(LocalDateTime.now());
        user.setUpdateTime(LocalDateTime.now());
        user.setCreater(UserContext.getUser());
        user.setUpdater(UserContext.getUser());
        save(user);
        return user;
    }

    /**
     * 实名认证
     */
    public void verifyCode(VerifyCodeDTO dto, Long userId) {
        // 验证身份证
        if (StringUtils.isNotBlank(dto.getIdentityCardId())) {
            if (!aliyunIdCardVerifier.verifyIdentityCard(dto.getIdentityCardId(), dto.getName())) {
                throw new RuntimeException("身份证验证失败");
            }
        } else {
            throw new RuntimeException("身份证号码不能为空");
        }
        lambdaUpdate()
                .eq(User::getId, userId)
                .set(User::getIdentityCardId, dto.getIdentityCardId())
                .update();
    }

    /**
     * 修改登录密码
     */
    public void updatePassword(UpdatePasswordDTO dto, Long userId) {
        User user = getById(userId);
        if (user == null) {
            throw new BadRequestException(USER_ID_NOT_EXISTS);
        }
        // 1.校验旧密码
        if (!user.getPassword().equals(dto.getOldPassword())) {
            throw new BadRequestException("密码错误");
        }
        // 2.修改密码
        user.setPassword(dto.getNewPassword());
        updateById(user);
    }

    /**
     * 绑定邮箱
     */
    public void updateEmail(UpdateEmailDTO dto, Long userId) {
        // 从Redis中获取验证码
        String storedCode = stringRedisTemplate.opsForValue().get("verificationCode:" + dto.getEmail());
        // 验证码校验
        if (storedCode == null || !storedCode.equals(dto.getCode())) {
            throw new RuntimeException("验证码错误或已过期");
        }
        // 验证码正确，清空Redis中的验证码
        stringRedisTemplate.delete("verificationCode:" + dto.getEmail());
        lambdaUpdate()
                .eq(User::getId, userId)
                .set(User::getEmail, dto.getEmail())
                .update();
    }

    /**
     * 统计用户任务数量
     */
    public UserTaskDTO getStats(Long userId) {
        List<UserTask> userTaskList = userTaskService.lambdaQuery().eq(UserTask::getUserId, userId).list();
        UserTaskDTO userTaskDTO = new UserTaskDTO();
        userTaskDTO.setOngoingTasks(0);
        userTaskDTO.setSettledTasks(0);

        Map<Integer, TaskMonth> tasks2024 = new HashMap<>();
        for (int i = 1; i <= 12; i++) {
            TaskMonth taskMonth = new TaskMonth();
            taskMonth.setTasksReceived(0);
            taskMonth.setTasksCompleted(0);
            tasks2024.put(i, taskMonth);
        }

        for (UserTask userTask : userTaskList) {
            // 获取任务创建月份
            LocalDateTime taskDateTime = userTask.getCreateTime();
            int month = taskDateTime.getMonthValue();

            if (userTask.getTaskStatus() == 2) {
                userTaskDTO.setOngoingTasks(userTaskDTO.getOngoingTasks() + 1);
                tasks2024.get(month).setTasksReceived(tasks2024.get(month).getTasksReceived() + 1);
            } else if (userTask.getTaskStatus() == 3) {
                userTaskDTO.setSettledTasks(userTaskDTO.getSettledTasks() + 1);
                tasks2024.get(month).setTasksReceived(tasks2024.get(month).getTasksReceived() + 1);
                tasks2024.get(month).setTasksCompleted(tasks2024.get(month).getTasksCompleted() + 1);
            }
        }
        userTaskDTO.setClaimedTasks(userTaskDTO.getOngoingTasks() + userTaskDTO.getSettledTasks());
        userTaskDTO.setTasks2024(tasks2024);

        List<Long> ids = userTaskList.stream()
                .filter(userTask -> userTask.getTaskStatus() == 3)
                .map(UserTask::getTaskId)
                .collect(Collectors.toList());
        List<NewTaskDTO> newTaskDTOS = taskClient.queryTaskById(ids);
        List<TaskDetail> taskDetails = BeanUtils.copyList(newTaskDTOS, TaskDetail.class);
        userTaskDTO.setTaskDetails(taskDetails);
        return userTaskDTO;
    }



    public User loginByPw(LoginFormDTO loginDTO) {
        // 1.数据校验
        String cellPhone = loginDTO.getCellPhone();
        if (StrUtil.isBlank(cellPhone)) {
            throw new BadRequestException(INVALID_UN);
        }
        // 2.根据手机号查询
        User user = lambdaQuery()
                .eq(StrUtil.isNotBlank(cellPhone), User::getCellPhone, cellPhone)
                .one();
        AssertUtils.isNotNull(user, INVALID_UN_OR_PW);
        // 3.校验是否禁用
        if (user.getStatus() == 0) {
            throw new ForbiddenException(USER_FROZEN);
        }
        // 4.校验密码
//        if (!passwordEncoder.matches(loginDTO.getPassword(), user.getPassword())) {
//            throw new BadRequestException(INVALID_UN_OR_PW);
//        }
        if (!loginDTO.getPassword().equals(user.getPassword())){
            throw new BadRequestException(INVALID_UN_OR_PW);
        }

        return user;
    }

    private Long handleRoleId(User user) {
        return 0L;
    }

    public User loginByVerifyCode(String phone, String code) {
        User user = lambdaQuery().eq(User::getCellPhone, phone).one();
        if (user == null) {
            UserRegisterDTO URDTO = new UserRegisterDTO();
            URDTO.setCode(code);
            URDTO.setCellPhone(phone);
            return phoneRegister(URDTO);
        }
        // 校验验证码
        codeService.verifyCode(phone, code);
        // 校验是否禁用
        if (user.getStatus() == 0) {
            throw new ForbiddenException(USER_FROZEN);
        }
        return user;
    }
}
